from flask import render_template, Blueprint, redirect, url_for, flash, request
from flask_login import login_required, current_user

from app import Project, messages, Shortlisting, db

bp = Blueprint("student", __name__, url_prefix="/student")


@bp.route("/projects")
@login_required
def projects():
    active_projects: list[Project] = Project.query.filter(
        Project.deleted.is_(None)
    ).all()
    short_listings = Shortlisting.query.filter_by(student=current_user.username).all()
    short_listings = {s.project_id for s in short_listings}
    return render_template(
        "pages/student/projects.html",
        projects=active_projects,
        short_listings=short_listings,
    )


@bp.route("/projects/shortlisted")
@login_required
def projects_ranking():
    shortlisted_projects: list[Project] = (
        Project.query.join(Shortlisting)
        .filter(Shortlisting.student == current_user.username)
        .order_by(Shortlisting.ranking)
        .all()
    )
    return render_template(
        "pages/student/shortlisted-projects.html", projects=shortlisted_projects
    )


@bp.route("/projects/rankings", methods=["PUT"])
@login_required
def update_ranking():
    project_ids = request.json
    current_shortlist: list[Shortlisting] = Shortlisting.query.filter(
        Shortlisting.project_id.in_(project_ids)
    ).filter_by(student=current_user.username)
    for shortlisting in current_shortlist:
        shortlisting.ranking = project_ids.index(shortlisting.project_id) + 1
    db.session.commit()
    return "", 204


@bp.route("/projects/<project_id>")
@login_required
def view_project(project_id):
    if project := Project.query.get(project_id):
        return render_template("pages/student/project-view.html", project=project)
    flash(messages.PROJECT_NOT_FOUND)
    return redirect(url_for("student.projects"))


@bp.route("/projects/<project_id>/shortlist")
@login_required
def shortlist_project(project_id):
    if Project.query.get(project_id):
        shortlisting = Shortlisting(
            student=current_user.username, project_id=project_id
        )
        db.session.add(shortlisting)
        db.session.commit()
    else:
        flash(messages.PROJECT_NOT_FOUND)
    return redirect(url_for("student.projects"))
